@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2019 ARM Limited.
+ * Copyright (c) 2018-2020 Arm Limited.
  *
  * SPDX-License-Identifier: MIT
  *
@@ -21,8 +21,8 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-#ifndef __ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H__
-#define __ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H__
+#ifndef ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H
+#define ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H
 
 #include "arm_compute/graph/Logger.h"
 #include "arm_compute/graph/Tensor.h"
@@ -47,6 +47,13 @@ namespace backends
 {
 namespace detail
 {
+// Address rule DR-9R5 (1579. Return by converting move constructor)
+#if defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5))
+#define RETURN_UNIQUE_PTR(x) (x)
+#else /* defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5)) */
+#define RETURN_UNIQUE_PTR(x) (std::move(x))
+#endif /* defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5)) */
+
 /** Returns backing tensor of a given tensor
  *
  * @tparam TargetInfo Target information
@@ -121,7 +128,7 @@ std::unique_ptr<IFunction> create_activation_layer(ActivationLayerNode &node)
                                << " InPlace : " << is_in_place_operation(input, output)
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend batch normalization layer function
@@ -165,7 +172,7 @@ std::unique_ptr<IFunction> create_batch_normalization_layer(BatchNormalizationLa
                                << " InPlace: " << is_in_place_operation(input, output)
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend batch normalization layer function
@@ -222,7 +229,7 @@ std::unique_ptr<IFunction> create_fused_convolution_batch_normalization_layer(Fu
                                << " Output shape: " << output->info()->tensor_shape()
                                << (fused_act.enabled() ? " " + to_string(fused_act.activation()) : "")
                                << std::endl);
-    return func;
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend fused depthwise convolution batch normalization layer function
@@ -278,7 +285,7 @@ std::unique_ptr<IFunction> create_fused_depthwise_convolution_batch_normalizatio
                                << " Output shape: " << output->info()->tensor_shape()
                                << (fused_act.enabled() ? " " + to_string(fused_act.activation()) : "")
                                << std::endl);
-    return func;
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend bounding box transform layer function
@@ -316,7 +323,7 @@ std::unique_ptr<IFunction> create_bounding_box_transform_layer(BoundingBoxTransf
                                << " BoundingBox Info img H: " << bbox_info.img_height() << " "
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend channel shuffle layer function
@@ -351,7 +358,7 @@ std::unique_ptr<IFunction> create_channel_shuffle_layer(ChannelShuffleLayerNode
                                << " Num groups: " << num_groups
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend layer concatenate function
@@ -376,7 +383,7 @@ std::unique_ptr<arm_compute::IFunction> create_concatenate_layer(ConcatenateLaye
     }
 
     // Extract IO and info
-    std::vector<typename TargetInfo::TensorType *> inputs;
+    std::vector<typename TargetInfo::SrcTensorType *> inputs;
     for(unsigned int i = 0; i < node.num_inputs(); ++i)
     {
         inputs.push_back(get_backing_tensor<TargetInfo>(node.input(i)));
@@ -407,7 +414,7 @@ std::unique_ptr<arm_compute::IFunction> create_concatenate_layer(ConcatenateLaye
                                << qss.str()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend convolution layer function
@@ -498,7 +505,7 @@ std::unique_ptr<IFunction> create_convolution_layer(ConvolutionLayerNode &node,
                                << qss.str()
                                << (fused_act.enabled() ? " " + to_string(fused_act.activation()) : "")
                                << std::endl);
-    return func;
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend deconvolution layer function
@@ -604,7 +611,7 @@ std::unique_ptr<IFunction> create_depthwise_convolution_layer(DepthwiseConvoluti
                                << qss.str()
                                << (fused_act.enabled() ? " " + to_string(fused_act.activation()) : "")
                                << std::endl);
-    return func;
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend dequantize layer function
@@ -643,7 +650,7 @@ std::unique_ptr<IFunction> create_dequantization_layer(DequantizationLayerNode &
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 /** Create a backend detection output layer function
  *
@@ -688,7 +695,7 @@ std::unique_ptr<IFunction> create_detection_output_layer(DetectionOutputLayerNod
                                << " DetectionOutputLayer info: " << detect_info
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend detection post process layer function
@@ -743,7 +750,7 @@ std::unique_ptr<IFunction> create_detection_post_process_layer(DetectionPostProc
                                << " DetectionPostProcessLayer info: " << detect_info
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend element-wise operation layer function
@@ -766,6 +773,7 @@ std::unique_ptr<IFunction> create_eltwise_layer(EltwiseLayerNode &node)
     typename TargetInfo::TensorType *output         = get_backing_tensor<TargetInfo>(node.output(0));
     const EltwiseOperation           eltwise_op     = node.eltwise_operation();
     const ConvertPolicy              convert_policy = node.convert_policy();
+    const ActivationLayerInfo        act_info       = node.fused_activation();
     ARM_COMPUTE_ERROR_ON(input1 == nullptr);
     ARM_COMPUTE_ERROR_ON(input2 == nullptr);
     ARM_COMPUTE_ERROR_ON(output == nullptr);
@@ -776,19 +784,19 @@ std::unique_ptr<IFunction> create_eltwise_layer(EltwiseLayerNode &node)
     {
         std::tie(func, func_name) = create_named_function<typename EltwiseFunctions::Addition>(
                                         std::string("ArithmeticAddition"),
-                                        input1, input2, output, convert_policy);
+                                        input1, input2, output, convert_policy, act_info);
     }
     else if(eltwise_op == EltwiseOperation::Sub)
     {
         std::tie(func, func_name) = create_named_function<typename EltwiseFunctions::Subtraction>(
                                         std::string("ArithmeticSubtraction"),
-                                        input1, input2, output, convert_policy);
+                                        input1, input2, output, convert_policy, act_info);
     }
     else if(eltwise_op == EltwiseOperation::Mul)
     {
         std::tie(func, func_name) = create_named_function<typename EltwiseFunctions::Multiplication>(
                                         std::string("PixelWiseMultiplication"),
-                                        input1, input2, output, 1.f, convert_policy, node.rounding_policy());
+                                        input1, input2, output, 1.f, convert_policy, node.rounding_policy(), act_info);
     }
     else
     {
@@ -805,7 +813,55 @@ std::unique_ptr<IFunction> create_eltwise_layer(EltwiseLayerNode &node)
                                << " Shape: " << input1->info()->tensor_shape()
                                << std::endl);
 
-    return func;
+    return RETURN_UNIQUE_PTR(func);
+}
+
+/** Create a backend unary element-wise operation layer function
+ *
+ * @tparam UnaryEltwiseFunctions Backend unary element-wise function
+ * @tparam TargetInfo       Target-specific information
+ *
+ * @param[in] node Node to create the backend function for
+ *
+ * @return Backend unary element-wise operation layer function
+ */
+template <typename UnaryEltwiseFunctions, typename TargetInfo>
+std::unique_ptr<IFunction> create_unary_eltwise_layer(UnaryEltwiseLayerNode &node)
+{
+    validate_node<TargetInfo>(node, 1 /* expected inputs */, 1 /* expected outputs */);
+
+    // Extract IO and info
+    typename TargetInfo::TensorType *input      = get_backing_tensor<TargetInfo>(node.input(0));
+    typename TargetInfo::TensorType *output     = get_backing_tensor<TargetInfo>(node.output(0));
+    const UnaryEltwiseOperation      eltwise_op = node.eltwise_descriptor().op;
+
+    ARM_COMPUTE_ERROR_ON(input == nullptr);
+    ARM_COMPUTE_ERROR_ON(output == nullptr);
+
+    std::unique_ptr<IFunction> func = nullptr;
+    std::string                func_name;
+    if(eltwise_op == UnaryEltwiseOperation::Exp)
+    {
+        std::tie(func, func_name) = create_named_function<typename UnaryEltwiseFunctions::Exp>(
+                                        std::string("Exp"),
+                                        input, output);
+    }
+    else
+    {
+        ARM_COMPUTE_ERROR("Unsupported unary element-wise operation!");
+    }
+
+    // Log info
+    ARM_COMPUTE_LOG_GRAPH_INFO("Instantiated "
+                               << node.name()
+                               << " Type: " << node.type()
+                               << " Target: " << TargetInfo::TargetType
+                               << " Operation: " << func_name
+                               << " Data Type: " << input->info()->data_type()
+                               << " Shape: " << input->info()->tensor_shape()
+                               << std::endl);
+
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend flatten layer function
@@ -843,7 +899,7 @@ std::unique_ptr<IFunction> create_flatten_layer(FlattenLayerNode &node)
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend fully connected layer function
@@ -899,7 +955,7 @@ std::unique_ptr<IFunction> create_fully_connected_layer(FullyConnectedLayerNode
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend generate proposals layer function
@@ -948,7 +1004,7 @@ std::unique_ptr<IFunction> create_generate_proposals_layer(GenerateProposalsLaye
                                << " Scores Out shape: " << scores_out->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend normalization layer function
@@ -990,7 +1046,7 @@ std::unique_ptr<IFunction> create_normalization_layer(NormalizationLayerNode &no
                                << " Normalization info: " << norm_info.type()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend normalize planar YUV layer function
@@ -1030,7 +1086,7 @@ std::unique_ptr<IFunction> create_normalize_planar_yuv_layer(NormalizePlanarYUVL
                                << " Shape: " << input->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend pad layer function
@@ -1048,15 +1104,16 @@ std::unique_ptr<IFunction> create_pad_layer(PadLayerNode &node)
     validate_node<TargetInfo>(node, 1 /* expected inputs */, 1 /* expected outputs */);
 
     // Extract IO and info
-    typename TargetInfo::TensorType *input   = get_backing_tensor<TargetInfo>(node.input(0));
-    typename TargetInfo::TensorType *output  = get_backing_tensor<TargetInfo>(node.output(0));
-    const PaddingList               &padding = node.padding();
+    typename TargetInfo::TensorType *input     = get_backing_tensor<TargetInfo>(node.input(0));
+    typename TargetInfo::TensorType *output    = get_backing_tensor<TargetInfo>(node.output(0));
+    const PaddingList               &padding   = node.padding();
+    const PixelValue                 pad_value = node.pad_value();
     ARM_COMPUTE_ERROR_ON(input == nullptr);
     ARM_COMPUTE_ERROR_ON(output == nullptr);
 
     // Create and configure function
     auto func = support::cpp14::make_unique<PadLayerFunction>();
-    func->configure(input, output, padding);
+    func->configure(input, output, padding, pad_value);
 
     // Log info
     ARM_COMPUTE_LOG_GRAPH_INFO("Instantiated "
@@ -1068,7 +1125,7 @@ std::unique_ptr<IFunction> create_pad_layer(PadLayerNode &node)
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend permute layer function
@@ -1107,7 +1164,7 @@ std::unique_ptr<IFunction> create_permute_layer(PermuteLayerNode &node)
                                << " Permutation vector: " << perm
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend pooling layer function
@@ -1143,10 +1200,77 @@ std::unique_ptr<IFunction> create_pooling_layer(PoolingLayerNode &node)
                                << " Data Type: " << input->info()->data_type()
                                << " Input shape: " << input->info()->tensor_shape()
                                << " Output shape: " << output->info()->tensor_shape()
-                               << " Pooling info: " << pool_info.pool_type()
+                               << " Pooling info: " << pool_info.pool_type
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
+}
+
+/** Create a backend PRelu layer function
+ *
+ * @tparam PReluFunction Backend PRelu function
+ * @tparam TargetInfo    Target-specific information
+ *
+ * @param[in] node Node to create the backend function for
+ *
+ * @return Backend PRelu layer function
+ */
+template <typename PReluFunction, typename TargetInfo>
+std::unique_ptr<IFunction> create_prelu_layer(PReluLayerNode &node)
+{
+    validate_node<TargetInfo>(node, 2 /* expected inputs */, 1 /* expected outputs */);
+
+    // Extract IO and info
+    typename TargetInfo::TensorType *input  = get_backing_tensor<TargetInfo>(node.input(0));
+    typename TargetInfo::TensorType *alpha  = get_backing_tensor<TargetInfo>(node.input(1));
+    typename TargetInfo::TensorType *output = get_backing_tensor<TargetInfo>(node.output(0));
+    ARM_COMPUTE_ERROR_ON(input == nullptr || alpha == nullptr);
+    ARM_COMPUTE_ERROR_ON(output == nullptr);
+
+    // Create and configure function
+    auto func = support::cpp14::make_unique<PReluFunction>();
+    func->configure(input, alpha, output);
+
+    // Log info
+    ARM_COMPUTE_LOG_GRAPH_INFO("Instantiated "
+                               << node.name()
+                               << " Type: " << node.type()
+                               << " Target: " << TargetInfo::TargetType
+                               << " Data Type: " << input->info()->data_type()
+                               << " Input shape: " << input->info()->tensor_shape()
+                               << " Output shape: " << output->info()->tensor_shape()
+                               << std::endl);
+
+    return RETURN_UNIQUE_PTR(func);
+}
+
+/** Create a backend print layer function
+ *
+ * @tparam TargetInfo Target-specific information
+ *
+ * @param[in] node Node to create the backend function for
+ *
+ * @return Backend print layer function
+ */
+template <typename TargetInfo>
+std::unique_ptr<IFunction> create_print_layer(PrintLayerNode &node)
+{
+    validate_node<TargetInfo>(node, 1 /* expected inputs */, 1 /* expected outputs */);
+
+    typename TargetInfo::TensorType *input = get_backing_tensor<TargetInfo>(node.input(0));
+    ARM_COMPUTE_ERROR_ON(input == nullptr);
+    ARM_COMPUTE_UNUSED(input);
+
+    // Log info
+    ARM_COMPUTE_LOG_GRAPH_INFO("Instantiated "
+                               << node.name()
+                               << " Type: " << node.type()
+                               << " Target: " << TargetInfo::TargetType
+                               << " Data Type: " << input->info()->data_type()
+                               << " Input shape: " << input->info()->tensor_shape()
+                               << std::endl);
+
+    return nullptr;
 }
 
 /** Create a backend priorbox layer function
@@ -1188,7 +1312,7 @@ std::unique_ptr<IFunction> create_priorbox_layer(PriorBoxLayerNode &node)
                                << " PriorBoxLayer info: " << prior_info
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend quantization layer function
@@ -1225,7 +1349,7 @@ std::unique_ptr<IFunction> create_quantization_layer(QuantizationLayerNode &node
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend reorg layer function
@@ -1262,7 +1386,7 @@ std::unique_ptr<IFunction> create_reorg_layer(ReorgLayerNode &node)
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend reshape layer function
@@ -1299,7 +1423,7 @@ std::unique_ptr<IFunction> create_reshape_layer(ReshapeLayerNode &node)
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend resize layer function
@@ -1325,7 +1449,7 @@ std::unique_ptr<IFunction> create_resize_layer(ResizeLayerNode &node)
 
     // Create and configure function
     auto func = support::cpp14::make_unique<ResizeLayerFunction>();
-    func->configure(input, output, policy, BorderMode::CONSTANT);
+    func->configure(input, output, ScaleKernelInfo{ policy, BorderMode::CONSTANT });
 
     // Log info
     ARM_COMPUTE_LOG_GRAPH_INFO("Instantiated "
@@ -1338,7 +1462,7 @@ std::unique_ptr<IFunction> create_resize_layer(ResizeLayerNode &node)
                                << " Interpolation: " << policy
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend ROI align layer function
@@ -1383,7 +1507,7 @@ std::unique_ptr<IFunction> create_roi_align_layer(ROIAlignLayerNode &node)
                                << " ROIPooling height: " << pool_info.pooled_height()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend slice layer function
@@ -1420,7 +1544,7 @@ std::unique_ptr<IFunction> create_slice_layer(SliceLayerNode &node)
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend softmax layer function
@@ -1459,7 +1583,7 @@ std::unique_ptr<IFunction> create_softmax_layer(SoftmaxLayerNode &node, GraphCon
                                << " Output shape: " << output->info()->tensor_shape()
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 
 /** Create a backend layer stack function
@@ -1502,7 +1626,7 @@ std::unique_ptr<arm_compute::IFunction> create_stack_layer(StackLayerNode &node)
                                << " Axis: " << axis
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 /** Create a backend Upsample layer function
  *
@@ -1546,7 +1670,7 @@ std::unique_ptr<IFunction> create_upsample_layer(UpsampleLayerNode &node, GraphC
                                << " Upsampling policy: " << upsampling_policy
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 /** Create a backend YOLO layer function
  *
@@ -1589,11 +1713,11 @@ std::unique_ptr<IFunction> create_yolo_layer(YOLOLayerNode &node, GraphContext &
                                << " Num classes: " << num_classes
                                << std::endl);
 
-    return std::move(func);
+    return RETURN_UNIQUE_PTR(func);
 }
 } // namespace detail
 } // namespace backends
 } // namespace graph
 } // namespace arm_compute
 
-#endif /* __ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H__ */
+#endif /* ARM_COMPUTE_GRAPH_BACKENDS_DETAIL_FUNCTION_HELPERS_H */
