@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2019 ARM Limited.
+ * Copyright (c) 2017-2020 Arm Limited.
  *
  * SPDX-License-Identifier: MIT
  *
@@ -64,135 +64,7 @@ TEST_SUITE(DepthwiseConvolutionLayer)
 
 // *INDENT-OFF*
 // clang-format off
-DATA_TEST_CASE(Validate3x3, framework::DatasetMode::ALL, zip(zip(zip(zip(zip(zip(zip(zip(
-               framework::dataset::make("InputInfo", { TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Mismatching data type input/weights
-                                                       TensorInfo(TensorShape(32U, 18U, 3U), 1, DataType::F32),     // Mismatching input feature maps
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Unsupported weights dimensions
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::QASYMM8), // Unsupported activation
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Mismatching depth multiplier
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Invalid stride
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Invalid biases size
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Invalid biases dimensions
-                                                       TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),     // Invalid output size
-                                                       TensorInfo(TensorShape(27U, 13U, 2U), 1, DataType::F32),     // Window shrink
-                                                       TensorInfo(TensorShape(32U, 18U, 8U), 1, DataType::F32),     // patch size bigger than input width
-                                                       TensorInfo(TensorShape(32U, 18U, 8U), 1, DataType::F32),     // dilation < 1
-                                                       TensorInfo(TensorShape(32U, 18U, 8U), 1, DataType::F32),
-                                                       TensorInfo(TensorShape(50U, 32U, 8U), 1, DataType::QASYMM8),
-                                                     }),
-               framework::dataset::make("WeightsInfo", { TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F16),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(5U, 5U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::QASYMM8),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 2U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 16U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 16U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 16U), 1, DataType::F32),
-                                                         TensorInfo(TensorShape(3U, 3U, 24U), 1, DataType::QASYMM8),
-                                                       })),
-               framework::dataset::make("BiasesInfo", { TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::S32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(4U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(24U), 1, DataType::S32),
-                                                      })),
-               framework::dataset::make("OutputInfo", { TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::QASYMM8),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(32U, 18U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(25U, 11U, 2U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(30U, 16U, 16U), 1, DataType::F32),
-                                                        TensorInfo(TensorShape(48U, 30U, 24U), 1, DataType::QASYMM8),
-                                                      })),
-               framework::dataset::make("ConvInfo", { PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(4, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                      PadStrideInfo(1, 1, 0, 0),
-                                                     })),
-               framework::dataset::make("DepthMultiplier", { 1,
-                                                             1,
-                                                             1,
-                                                             1,
-                                                             3,
-                                                             1,
-                                                             1,
-                                                             1,
-                                                             1,
-                                                             1,
-                                                             2,
-                                                             2,
-                                                             2,
-                                                             3,
-                                                            })),
-                framework::dataset::make("ActivationInfo", { ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(ActivationLayerInfo::ActivationFunction::LINEAR),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(),
-                                                             ActivationLayerInfo(ActivationLayerInfo::ActivationFunction::RELU),
-                                                           })),
-                framework::dataset::make("Dilation", { Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(20U, 1U),
-                                                       Size2D(0U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       Size2D(1U, 1U),
-                                                       })),
-               framework::dataset::make("Expected", { false, false, false, false, false, false, false, false, false, false, false, false, true, true })),
-               input_info, weights_info, biases_info, output_info, conv_info, depth_multiplier, act_info, dilation, expected)
-{
-    bool is_valid = bool(CLDepthwiseConvolutionLayer3x3::validate(&input_info.clone()->set_is_resizable(false), &weights_info.clone()->set_is_resizable(false), &biases_info.clone()->set_is_resizable(false), &output_info.clone()->set_is_resizable(false), conv_info, depth_multiplier, act_info,GPUTarget::MIDGARD, dilation));
-    ARM_COMPUTE_EXPECT(is_valid == expected, framework::LogLevel::ERRORS);
-}
-
-DATA_TEST_CASE(ValidateGeneric, framework::DatasetMode::ALL, zip(zip(zip(zip(zip(zip(zip(
+DATA_TEST_CASE(Validate, framework::DatasetMode::ALL, zip(zip(zip(zip(zip(zip(zip(
                 framework::dataset::make("InputInfo", { TensorInfo(TensorShape(27U, 13U, 2U), 1, DataType::F32),    // Mismatching data type input/weights
                                                         TensorInfo(TensorShape(27U, 13U, 3U), 1, DataType::F32),    // Mismatching input feature maps
                                                         TensorInfo(TensorShape(27U, 13U, 2U), 1, DataType::F32),    // Mismatching depth multiplier
@@ -563,9 +435,9 @@ FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseConvolutionLayerDataset(),
                                                                        depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.3f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.3f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 4) })),
-                                       framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
+                                       framework::dataset::make("DataLayout", { DataLayout::NHWC })), // NCHW is tested with int8
                                ActivationFunctionsDataset))
 {
     validate(CLAccessor(_target), _reference, tolerance_qasymm8);
@@ -574,7 +446,7 @@ FIXTURE_DATA_TEST_CASE(RunLarge, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::LargeDepthwiseConvolutionLayerDataset(),
                                                                        large_depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.7f, 2) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -586,9 +458,9 @@ FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseDilatedConvolutionLayerDataset(),
                                                                        depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.8, 1) })),
-                                       framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
+                                       framework::dataset::make("DataLayout", { DataLayout::NHWC })), // NCHW is tested with int8
                                ActivationFunctionsDataset))
 {
     validate(CLAccessor(_target), _reference, tolerance_qasymm8);
@@ -597,7 +469,7 @@ FIXTURE_DATA_TEST_CASE(RunLarge, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::LargeDepthwiseDilatedConvolutionLayerDataset(),
                                                                        large_depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(1.3f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.9f, 11) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -611,7 +483,7 @@ FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseConvolutionLayerDataset3x3(),
                                                                        depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.3f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.3f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -622,7 +494,7 @@ FIXTURE_DATA_TEST_CASE(RunLarge, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::LargeDepthwiseConvolutionLayerDataset3x3(),
                                                                        large_depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -634,7 +506,7 @@ FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseDilatedConvolutionLayerDataset3x3(),
                                                                        depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -645,7 +517,7 @@ FIXTURE_DATA_TEST_CASE(RunLarge, CLDepthwiseConvolutionLayerQuantizedFixture<uin
                        combine(combine(combine(combine(combine(combine(datasets::LargeDepthwiseDilatedConvolutionLayerDataset3x3(),
                                                                        large_depth_multipliers),
                                                                framework::dataset::make("DataType", DataType::QASYMM8)),
-                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
                                                framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 10) })),
                                        framework::dataset::make("DataLayout", { DataLayout::NCHW, DataLayout::NHWC })),
                                ActivationFunctionsDataset))
@@ -656,6 +528,35 @@ TEST_SUITE_END() // Dilation
 TEST_SUITE_END() // W3x3
 TEST_SUITE_END() // QASYMM8
 
+TEST_SUITE(QASYMM8_SIGNED)
+TEST_SUITE(Generic)
+FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<int8_t>, framework::DatasetMode::PRECOMMIT,
+                       combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseConvolutionLayerDataset(),
+                                                                       depth_multipliers),
+                                                               framework::dataset::make("DataType", DataType::QASYMM8_SIGNED)),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.3f, 10), QuantizationInfo(2.2f, 10) })),
+                                               framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.5f, 4) })),
+                                       framework::dataset::make("DataLayout", { DataLayout::NCHW })),
+                               ActivationFunctionsDataset))
+{
+    validate(CLAccessor(_target), _reference, tolerance_qasymm8);
+}
+TEST_SUITE(Dilation)
+FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedFixture<int8_t>, framework::DatasetMode::PRECOMMIT,
+                       combine(combine(combine(combine(combine(combine(datasets::SmallDepthwiseDilatedConvolutionLayerDataset(),
+                                                                       depth_multipliers),
+                                                               framework::dataset::make("DataType", DataType::QASYMM8_SIGNED)),
+                                                       framework::dataset::make("SrcQuantizationInfo", { QuantizationInfo(0.5f, 10), QuantizationInfo(2.2f, 10) })),
+                                               framework::dataset::make("DstQuantizationInfo", { QuantizationInfo(0.8, 1) })),
+                                       framework::dataset::make("DataLayout", { DataLayout::NCHW })),
+                               ActivationFunctionsDataset))
+{
+    validate(CLAccessor(_target), _reference, tolerance_qasymm8);
+}
+TEST_SUITE_END() // Dilation
+TEST_SUITE_END() // Generic
+TEST_SUITE_END() // QASYMM8_SIGNED
+
 TEST_SUITE(QSYMM8_PER_CHANNEL)
 TEST_SUITE(Generic)
 FIXTURE_DATA_TEST_CASE(RunSmall, CLDepthwiseConvolutionLayerQuantizedPerChannelFixture<uint8_t>, framework::DatasetMode::PRECOMMIT,
